// --- [ usi_klingel.c ] ------------------------------------------------------
/*       08.01.09
*        Michael S.
* Ein ATTINY85 bildet ein USI-Slave, der eine Klingel/Rechtecksoundgenerator steuert.
* bentigt ca. 5100 Byte, davon sind ca. 3600 Byte Melodiedaten (geschtzt)
*
* Der Aufruf: TWI_ADR, Liednr
* Prfen, ob die Klingel gerade aktiv ist: TWI_ADR+1, 1 Byte lesen: wenn > 0, dann aktiv
* Es wird die Nr. des Liedes zurckgegeben, das gerade abluft
*
* Alternativ bergabe eines eigenen Liedes:
*
* Der Aufruf: TWI_ADR, Lnge des Liedes, Geschwindigkeit, Noten .......
*   Beispiel: 64,7,5,10,12,10,12,10 -> langsame Folge wie Tattata
*             64,7,1,10,12,10,12,10 -> schnelle Folge wie Klingelsignal
*
* Die TWI-Adresse kann von aussen gendert werden per General Call, alte Adresse, neue Adresse
* Die neue Adresse wird dauerhaft im Eeprom gespeichert.
*  Beispiel: 0,64,8			
*
* Die Klingel selbst stammt von M. Baudisch (Dez. 2008)
*
* Tabwidth: 3
*/
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <inttypes.h>
#include <util/delay.h>
#include "usi_twi_slave.h"

volatile uint8_t General_Call;
uint8_t buffer[TWI_RX_BUFFER_SIZE];	// den Buffer gleich gro machen wie den Rx_Buffer

extern void klingel_init(void);
extern void klingel(uint8_t liednr);

// ----------------------------------------------------------------------------
int main( void )
{
register uint8_t j;						// Zhler

General_Call = FALSE;

MCUCR |= (1<<PUD);						// Pullups disablen
//DDRB   = (1<<PB1);						// Ausgangspin fr Sound - hier berflssig

												// Stromsparmodus , aktiviert direkt vor sleep
MCUCR |= (1<<SM1 | 0<< SM0 | 1<<SE);// Powerdown-Modus, Sleep enable

// Timer0, Timer1 und USI werden bentigt !
PRR   = (1<<PRADC);						// ADC disablen
ACSR  = (1<<ACD);							// Analog-Comparator disable

CLKPR = (1<<CLKPCE);						// auf 8 MHz schalten
CLKPR = 0;

sei();	 									// enable_interrupt

usi_twi_slave_init();					// den Slave und seine Adresse initialisieren
												// set USI_State = USI_WAIT
klingel_init();							// Timer

while(1)										// Endlosschleife
	{											// warten, bis eine laufende Transaktion beendet ist
	while (USI_State & 0xF0);
	if (USI_State)  						// wenn ein STOP oder NACK vorliegt (jedoch kein USI_WAIT!)
		{										// bei USI_WAIT wird direkt in den SLEEP-Modus gewechselt
		USI_State = USI_WAIT;			// Flag zurcksetzen
		if (General_Call)
			{
			General_Call = FALSE;		// General Call Flag zurcksetzen
			eeprom_write_twi_adr();		// gewnschte Adresse prfen und ins Eeprom schreiben
			}

		else									// wenn kein General_Call
			{
			j = usi_twi_rx_bytes(); 	// Anzahl der empfangenen Bytes holen
												// und den Puffer auslesen
			while(j)							// eine Schleife aufbauen fr den Fall, dass noch ein Klingeljob folgt
				{
				usi_twi_get_data(buffer, j);
				usi_twi_put_data(buffer, 1); // Zum Testen, ob die Klingel aktiv ist, 
												// wird das 1. Byte in den Ausgangspuffer kopiert
												// wenn nun die Klingel ausgelesen wird, dann liefert
												// das erste Byte einen Wert > 0 = Klingel ist aktiv
												// das erste Byte ist die Liednr oder die Lnge des bergebenen Liedes
												
												// wenn genau 1 Byte bergeben wurde
				if (j == 1) 				// dann ist das die Liednr., die steht in buffer[0]
					{
					if (buffer[0] < 101) klingel(buffer[0]); // berprfung auf 0 macht klingel() selbst
					}
				else							// wenn mehr als 1 Byte bergeben wurde, dann sollte das 
					{							// ein Lied sein, dass in buffer[] steht
												// die angegebene Lnge des Liedes darf aber nicht lnger
												// sein als das Array TWI_RxBuf[]
					if (buffer[0] > TWI_RX_BUFFER_SIZE) buffer[0] = TWI_RX_BUFFER_SIZE;
					klingel(101);			// jetzt wird klingel() mit dem Parameter 101 aufgerufen
					}							// 
			
				while (USI_State & 0xF0);// falls gerade ein neuer Job ankommt
				j = usi_twi_rx_bytes();	// prfen, ob zwischenzeitlich ein neuer Job angekommen ist,
												// es wird nur der letzte bercksichtigt
				}								// while(j) - solange noch ein Job ansteht
			
//			for (j = 0; j < TWI_RX_BUFFER_SIZE; j++) buffer[j] = 0;

			buffer[0] = 0;					// buffer[0] = 0 in den Tx_Buffer schreiben
			usi_twi_put_data(buffer, 1);
												// wenn man die Klingel ausliest, dann liefert
												// das erste Byte ein 0 zurck = Klingel ist aus
			}									// if (General_Call)
		}										// if (USI_State) - es gibt kein else
		asm volatile ("sleep");			// in Sleep-Modus verfallen
	}											// while(1)
}

// --- [ eof ] ----------------------------------------------------------------

